@@ -1,7 +1,6 @@
 /*
  * qqwing - Sudoku solver and generator
- * Copyright (C) 2006-2014 Stephen Ostermiller
- * http://ostermiller.org/qqwing/
+ * Copyright (C) 2006-2014 Stephen Ostermiller http://ostermiller.org/
  * Copyright (C) 2007 Jacques Bensimon (jacques@ipm.com)
  * Copyright (C) 2011 Jean Guillerez (j.guillerez - orange.fr)
  * Copyright (C) 2014 Michael Catanzaro (mcatanzaro@gnome.org)
@@ -58,7 +57,7 @@ namespace qqwing {
 				COLUMN_BOX,
 				HIDDEN_PAIR_ROW,
 				HIDDEN_PAIR_COLUMN,
-				HIDDEN_PAIR_SECTION,
+				HIDDEN_PAIR_SECTION
 			};
 			LogItem(int round, LogType type);
 			LogItem(int round, LogType type, int value, int position);
@@ -109,17 +108,20 @@ namespace qqwing {
 	/**
 	 * Create a new Sudoku board
 	 */
-	SudokuBoard::SudokuBoard(){
-		puzzle = new int[BOARD_SIZE];
-		solution = new int[BOARD_SIZE];
-		solutionRound = new int[BOARD_SIZE];
-		possibilities = new int[POSSIBILITY_SIZE];
-		recordHistory = false;
-		printStyle = READABLE;
-		randomBoardArray = new int[BOARD_SIZE];
-		randomPossibilityArray = new int[ROW_COL_SEC_SIZE];
-		solveHistory = new vector<LogItem*>();
-		solveInstructions = new vector<LogItem*>();
+	SudokuBoard::SudokuBoard() :
+		puzzle ( new int[BOARD_SIZE] ),
+		solution ( new int[BOARD_SIZE] ),
+		solutionRound ( new int[BOARD_SIZE] ),
+		possibilities ( new int[POSSIBILITY_SIZE] ),
+		randomBoardArray ( new int[BOARD_SIZE] ),
+		randomPossibilityArray ( new int[ROW_COL_SEC_SIZE] ),
+		recordHistory ( false ),
+		logHistory( false ),
+		solveHistory ( new vector<LogItem*>() ),
+		solveInstructions ( new vector<LogItem*>() ),
+		printStyle ( READABLE ),
+		lastSolveRound (0)
+	{
 		{for (int i=0; i<BOARD_SIZE; i++){
 			randomBoardArray[i] = i;
 		}}
@@ -183,7 +185,7 @@ namespace qqwing {
 			possibilities[i] = 0;
 		}}
 
-		{for (int i=0;i<solveHistory->size();i++){
+		{for (unsigned int i=0; i<solveHistory->size(); i++){
 			delete solveHistory->at(i);
 		}}
 		solveHistory->clear();
@@ -223,11 +225,11 @@ namespace qqwing {
 	string SudokuBoard::getDifficultyAsString(){
 		SudokuBoard::Difficulty difficulty = getDifficulty();
 		switch (difficulty){
-			case SudokuBoard::EXPERT: return "Expert"; break;
-			case SudokuBoard::INTERMEDIATE: return "Intermediate"; break;
-			case SudokuBoard::EASY: return "Easy"; break;
-			case SudokuBoard::SIMPLE: return "Simple"; break;
-			default: return "Unknown"; break;
+			case SudokuBoard::EXPERT: return "Expert";
+			case SudokuBoard::INTERMEDIATE: return "Intermediate";
+			case SudokuBoard::EASY: return "Easy";
+			case SudokuBoard::SIMPLE: return "Simple";
+			default: return "Unknown";
 		}
 	}
 
@@ -384,6 +386,10 @@ namespace qqwing {
 					case FLIP:
 						positionsym1 = rowColumnToCell(ROW_COL_SEC_SIZE-1-cellToRow(position),cellToColumn(position));
 					break;
+					case RANDOM: // NOTE: Should never happen
+					break;
+					case NONE: // NOTE: No need to do anything
+					break;
 				}
 				// try backing out the value and
 				// counting solutions to the puzzle
@@ -468,7 +474,7 @@ namespace qqwing {
 				cout << endl;
 			}
 		}
-		{for (int i=0;i<v->size();i++){
+		{for (unsigned int i=0;i<v->size();i++){
 			cout << i+1 << ". ";
 			v->at(i)->print();
 			if (printStyle == CSV){
@@ -523,7 +529,19 @@ namespace qqwing {
 		return false;
 	}
 
+	bool SudokuBoard::hasUniqueSolution(){
+		return countSolutionsLimited() == 1;
+	}
+
 	int SudokuBoard::countSolutions(){
+		return countSolutions(false);
+	}
+
+	int SudokuBoard::countSolutionsLimited(){
+		return countSolutions(true);
+	}
+
+	int SudokuBoard::countSolutions(bool limitToTwo){
 		// Don't record history while generating.
 		bool recHistory = recordHistory;
 		setRecordHistory(false);
@@ -531,7 +549,7 @@ namespace qqwing {
 		setLogHistory(false);
 
 		reset();
-		int solutionCount = countSolutions(2, false);
+		int solutionCount = countSolutions(2, limitToTwo);
 
 		// Restore recording history.
 		setRecordHistory(recHistory);
@@ -1377,12 +1395,12 @@ namespace qqwing {
 
 	SudokuBoard::~SudokuBoard(){
 		clearPuzzle();
-		delete puzzle;
-		delete solution;
-		delete possibilities;
-		delete solutionRound;
-		delete randomBoardArray;
-		delete randomPossibilityArray;
+		delete[] puzzle;
+		delete[] solution;
+		delete[] possibilities;
+		delete[] solutionRound;
+		delete[] randomBoardArray;
+		delete[] randomPossibilityArray;
 		delete solveHistory;
 		delete solveInstructions;
 	}
@@ -1500,8 +1518,8 @@ namespace qqwing {
 	 * log items in the vector are of the specified type.
 	 */
 	int getLogCount(vector<LogItem*>* v, LogItem::LogType type){
-		int count = 0;
-		{for (int i=0; i<v->size(); i++){
+		unsigned int count = 0;
+		{for (unsigned int i=0; i<v->size(); i++){
 			if(v->at(i)->getType() == type) count++;
 		}}
 		return count;
@@ -1527,6 +1545,7 @@ namespace qqwing {
 			case 2: return SudokuBoard::MIRROR;
 			case 3: return SudokuBoard::FLIP;
 		}
+		return SudokuBoard::ROTATE90; // NOTE: default action
 	}
 
 	/**
